@@ -1,7 +1,7 @@
 # Locale::Po4a::Po -- manipulation of PO files
 #
 # This program is free software; you may redistribute it and/or modify it
-# under the terms of GPL (see COPYING).
+# under the terms of GPL v2.0 or later (see COPYING).
 
 ############################################################################
 # Modules and declarations
@@ -116,7 +116,7 @@ use Locale::Po4a::TransTractor;
 # Try to use a C extension if present.
 eval("bootstrap Locale::Po4a::Po $Locale::Po4a::TransTractor::VERSION");
 
-use 5.006;
+use 5.16.0;
 use strict;
 use warnings;
 
@@ -225,7 +225,6 @@ sub initialize {
     $self->{options}{'package-name'}       = "PACKAGE";
     $self->{options}{'package-version'}    = "VERSION";
     $self->{options}{'wrap-po'}            = 76;
-    $self->{options}{'pot-charset'}        = "UTF-8";
     $self->{options}{'pot-language'}       = "";
 
     foreach my $opt ( keys %$options ) {
@@ -260,11 +259,14 @@ sub initialize {
         $self->{counter} = {};
     }
 
-    $self->{po}        = ();
-    $self->{count}     = 0;                          # number of msgids in the PO
+    $self->{po}               = ();
+    $self->{count}            = 0;                   # number of msgids in the PO
                                                      # count_doc: number of strings in the document
                                                      # (duplicate strings counted multiple times)
-    $self->{count_doc} = 0;
+    $self->{count_doc}        = 0;
+    $self->{gettextize_types} = ();                  # Type of each msgid found in the doc, in order
+        # We cannot use {$msgid}{'type'} as a type because for duplicate entries, the type is overwritten.
+        # So we have to copy the same info to this separate array, which is accessed through type_doc()
     $self->{header_comment} =
         " SOME DESCRIPTIVE TITLE\n"
       . " Copyright (C) YEAR "
@@ -293,13 +295,11 @@ sub initialize {
           . "Language: "
           . $self->{options}{'pot-language'} . "\n"
           . "MIME-Version: 1.0\n"
-          . "Content-Type: text/plain; charset="
-          . $self->{options}{'pot-charset'} . "\n"
+          . "Content-Type: text/plain; charset=UTF-8\n"
           . "Content-Transfer-Encoding: 8bit\n"
     );
 
-    $self->{encoder} = find_encoding("UTF-8");
-    $self->{footer}  = [];
+    $self->{footer} = [];
 
     # To make stats about gettext hits
     $self->stats_clear();
@@ -318,39 +318,73 @@ sub read {
     my $filename = shift
       or croak wrap_mod( "po4a::po", dgettext( "po4a", "Please provide a non-null filename" ) );
 
+    my $charset = shift // 'UTF-8';
+    $charset = 'UTF-8' if $charset eq "CHARSET";
+    warn "Read $filename with encoding: $charset" if $debug{'encoding'};
+
+    my $checkvalidity = shift // 1;
+
     my $lang = basename($filename);
     $lang =~ s/\.po$//;
     $self->{lang} = $lang;
 
-    my $cmd = "msgfmt" . $Config{_exe} . " --check-format --check-domain -o /dev/null " . $filename;
+    if ($checkvalidity) {   # We sometimes need to read a file even if it may be invalid (eg to test whether it's empty)
+        my $cmd = "msgfmt" . $Config{_exe} . " --check-format --check-domain -o /dev/null \"" . $filename . '"';
 
-    my $locale = $ENV{'LC_ALL'};
-    $ENV{'LC_ALL'} = "C";
-    my $out = qx/$cmd 2>&1/;
-    $ENV{'LC_ALL'} = $locale;
+        my $locale = $ENV{'LC_ALL'};
+        $ENV{'LC_ALL'} = "C";
+        my $out = qx/$cmd 2>&1/;
+        $ENV{'LC_ALL'} = $locale;
 
-    die wrap_msg( dgettext( "po4a", "Invalid po file %s:\n%s" ), $filename, $out )
-      unless ( $? == 0 );
+        die wrap_msg( dgettext( "po4a", "Invalid po file %s:\n%s" ), $filename, $out )
+          unless ( $? == 0 );
+    }
 
     my $fh;
     if ( $filename eq '-' ) {
         $fh = *STDIN;
     } else {
-        open $fh, "<$filename"
+        open( $fh, "<:encoding($charset)", $filename )
           or croak wrap_mod( "po4a::po", dgettext( "po4a", "Cannot read from %s: %s" ), $filename, $! );
     }
 
     ## Read paragraphs line-by-line
     my $pofile = "";
-    my $textline;
-    while ( defined( $textline = <$fh> ) ) {
+    while ( defined( my $textline = <$fh> ) ) {
         $pofile .= $textline;
     }
+    $pofile =~ s/\r\n/\n/sg;    # Reading a DOS-encoded file from Linux (native files are handled in all cases)
 
-    #    close INPUT
-    #        or croak (sprintf(dgettext("po4a",
-    #                                   "Cannot close %s after reading: %s"),
-    #                          $filename,$!)."\n");
+    # If we did not get the charset right, reload the file with the right one
+    if ( $pofile =~ /charset=(.*?)[\s\\]/ ) {
+        my $detected_charset = $1;
+
+        if ( $detected_charset ne $charset && uc($detected_charset) ne $charset && uc($detected_charset) ne 'CHARSET' )
+        {
+            warn "Reloading the PO file, changing the charset from '$charset' to '$detected_charset'"
+              if $debug{'encoding'};
+            $self->read( $filename, $detected_charset, $checkvalidity );
+            return;
+        }
+    }
+
+    if ( $pofile =~ m/^\N{BOM}/ ) {    # UTF-8 BOM detected
+        croak "BOM detected";
+        croak wrap_msg(
+            dgettext(
+                "po4a",
+                "The file %s starts with a BOM char indicating that its encoding is UTF-8, but you specified %s instead."
+            ),
+            $filename,
+            $charset
+        ) if ( uc($charset) ne 'UTF-8' );
+        $pofile =~ s/^\N{BOM}//;
+    }
+
+    if ( $filename ne '-' ) {
+        close $fh
+          or croak wrap_mod( "po4a::po", dgettext( "po4a", "Cannot close %s after reading: %s" ), $filename, $! );
+    }
 
     my $linenum = 0;
 
@@ -503,15 +537,27 @@ sub write {
             File::Path::mkpath( $dir, 0, 0755 )    # Croaks on error
               if ( length($dir) && !-e $dir );
         }
-        open $fh, ">$filename"
+        open( $fh, '>:encoding(UTF-8)', $filename )
           or croak wrap_mod( "po4a::po", dgettext( "po4a", "Cannot write to %s: %s" ), $filename, $! );
     }
 
     print $fh "" . format_comment( $self->{header_comment}, "" )
       if length( $self->{header_comment} );
 
+    # Force the encoding of PO files in UTF-8 on disk, because msgmerge can get messed up when mixing encodings
+    # See https://savannah.gnu.org/bugs/index.php?65104
+    my $header = $self->{header};
+    $header =~ /charset=([^\s\\]*)/i;
+    my $oldcharset = $1 // '';
+    # TODO: rewrite to "msgmerge suffers some bugs when PO files are not encoded in UTF-8; Recoding %s to UTF-8 (was %s) to circumvent the issue"
+    warn sprintf(
+        dgettext( "po4a", "Force the encoding of %s to UTF-8 (was %s), as mixing encodings may break msgmerge.\n" ),
+        $filename, $oldcharset )
+      if $oldcharset ne 'UTF-8';
+    $header =~ s/charset=[^\s\\]*/charset=UTF-8/i;
+
     print $fh "msgid \"\"\n";
-    print $fh "msgstr " . quote_text( $self->{header}, $self->{options}{'wrap-po'} ) . "\n\n";
+    print $fh "msgstr " . quote_text( $header, $self->{options}{'wrap-po'} ) . "\n\n";
 
     my $buf_msgstr_plural;    # Used to keep the first msgstr of plural forms
     my $first = 1;
@@ -546,62 +592,32 @@ sub write {
 
         if ( exists $self->{po}{$msgid}{'plural'} ) {
             if ( $self->{po}{$msgid}{'plural'} == 0 ) {
-                if ( $self->get_charset =~ /^utf-8$/i ) {
-                    my $msgstr = Encode::decode_utf8( $self->{po}{$msgid}{'msgstr'} );
-                    $msgid = Encode::decode_utf8($msgid);
-                    $output .=
-                      Encode::encode_utf8( "msgid " . quote_text( $msgid, $self->{options}{'wrap-po'} ) . "\n" );
-                    $buf_msgstr_plural =
-                      Encode::encode_utf8( "msgstr[0] " . quote_text( $msgstr, $self->{options}{'wrap-po'} ) . "\n" );
-                } else {
-                    $output = "msgid " . quote_text( $msgid, $self->{options}{'wrap-po'} ) . "\n";
-                    $buf_msgstr_plural =
-                      "msgstr[0] " . quote_text( $self->{po}{$msgid}{'msgstr'}, $self->{options}{'wrap-po'} ) . "\n";
-                }
+                $output .= "msgid " . quote_text( $msgid, $self->{options}{'wrap-po'} ) . "\n";
+                $buf_msgstr_plural =
+                  "msgstr[0] " . quote_text( $self->{po}{$msgid}{'msgstr'}, $self->{options}{'wrap-po'} ) . "\n";
             } elsif ( $self->{po}{$msgid}{'plural'} == 1 ) {
 
                 # TODO: there may be only one plural form
-                if ( $self->get_charset =~ /^utf-8$/i ) {
-                    my $msgstr = Encode::decode_utf8( $self->{po}{$msgid}{'msgstr'} );
-                    $msgid = Encode::decode_utf8($msgid);
-                    $output =
-                      Encode::encode_utf8( "msgid_plural " . quote_text( $msgid, $self->{options}{'wrap-po'} ) . "\n" );
-                    $output .= $buf_msgstr_plural;
-                    $output .=
-                      Encode::encode_utf8( "msgstr[1] " . quote_text( $msgstr, $self->{options}{'wrap-po'} ) . "\n" );
-                    $buf_msgstr_plural = "";
-                } else {
-                    $output = "msgid_plural " . quote_text( $msgid, $self->{options}{'wrap-po'} ) . "\n";
-                    $output .= $buf_msgstr_plural;
-                    $output .=
-                      "msgstr[1] " . quote_text( $self->{po}{$msgid}{'msgstr'}, $self->{options}{'wrap-po'} ) . "\n";
-                }
+                $output = "msgid_plural " . quote_text( $msgid, $self->{options}{'wrap-po'} ) . "\n";
+                $output .= $buf_msgstr_plural;
+                $output .=
+                  "msgstr[1] " . quote_text( $self->{po}{$msgid}{'msgstr'}, $self->{options}{'wrap-po'} ) . "\n";
             } else {
                 die wrap_msg( dgettext( "po4a", "Cannot write PO files with more than two plural forms." ) );
             }
         } else {
-            if ( $self->get_charset =~ /^utf-8$/i ) {
-                my $msgstr = Encode::decode_utf8( $self->{po}{$msgid}{'msgstr'} );
-                $msgid = Encode::decode_utf8($msgid);
-                $output .= Encode::encode_utf8( "msgid " . quote_text( $msgid, $self->{options}{'wrap-po'} ) . "\n" );
-                $output .= Encode::encode_utf8( "msgstr " . quote_text( $msgstr, $self->{options}{'wrap-po'} ) . "\n" );
-            } else {
-                $output .= "msgid " . quote_text( $msgid, $self->{options}{'wrap-po'} ) . "\n";
-                $output .= "msgstr " . quote_text( $self->{po}{$msgid}{'msgstr'}, $self->{options}{'wrap-po'} ) . "\n";
-            }
+            $output .= "msgid " . quote_text( $msgid, $self->{options}{'wrap-po'} ) . "\n";
+            $output .= "msgstr " . quote_text( $self->{po}{$msgid}{'msgstr'}, $self->{options}{'wrap-po'} ) . "\n";
         }
 
         print $fh $output;
     }
     print $fh join( "\n\n", @{ $self->{footer} } ) if scalar @{ $self->{footer} };
 
-    #    print STDERR "$fh";
-    #    if ($filename ne '-') {
-    #        close $fh
-    #            or croak (sprintf(dgettext("po4a",
-    #                                       "Cannot close %s after writing: %s\n"),
-    #                              $filename,$!));
-    #    }
+    if ( $filename ne '-' ) {
+        close $fh
+          or croak wrap_mod( dgettext( "po4a", "Cannot close %s after writing: %s\n" ), $filename, $! );
+    }
 }
 
 =item write_if_needed($$)
@@ -646,7 +662,7 @@ sub write_if_needed {
         my $basename = basename($filename);
         ( undef, $tmp_filename ) = File::Temp::tempfile(
             $basename . "XXXX",
-            DIR    => $ENV{TMPDIR} || "/tmp",
+            DIR    => File::Spec->tmpdir(),
             OPEN   => 0,
             UNLINK => 0
         );
@@ -657,169 +673,6 @@ sub write_if_needed {
     }
 }
 
-=item gettextize($$)
-
-This function produces one translated message catalog from two catalogs, an
-original and a translation. This process is described in L<po4a(7)|po4a.7>,
-section I<Gettextization: how does it work?>.
-
-=cut
-
-sub gettextize {
-    my $this  = shift;
-    my $class = ref($this) || $this;
-    my ( $poorig, $potrans ) = ( shift, shift );
-
-    my $pores = Locale::Po4a::Po->new();
-
-    my $please_fail = 0;
-    my $toobad      = dgettext( "po4a",
-            "\nThe gettextization failed (once again). Don't give up, "
-          . "gettextizing is a subtle art, but this is only needed once "
-          . "to convert a project to the gorgeous luxus offered by po4a "
-          . "to translators."
-          . "\nPlease refer to the po4a(7) documentation, the section "
-          . "\"HOWTO convert a pre-existing translation to po4a?\" "
-          . "contains several hints to help you in your task" );
-
-    # Don't fail right now when the entry count does not match. Instead, give
-    # it a try so that the user can see where we fail (which is probably where
-    # the problem is).
-    if ( $poorig->count_entries_doc() > $potrans->count_entries_doc() ) {
-        warn wrap_mod(
-            "po4a gettextize",
-            dgettext(
-                "po4a",
-                "Original has more strings than the translation (%d>%d). "
-                  . "Please fix it by editing the translated version to add "
-                  . "some dummy entry."
-            ),
-            $poorig->count_entries_doc(),
-            $potrans->count_entries_doc()
-        );
-        $please_fail = 1;
-    } elsif ( $poorig->count_entries_doc() < $potrans->count_entries_doc() ) {
-        warn wrap_mod(
-            "po4a gettextize",
-            dgettext(
-                "po4a",
-                "Original has less strings than the translation (%d<%d). "
-                  . "Please fix it by removing the extra entry from the "
-                  . "translated file. You may need an addendum (cf po4a(7)) "
-                  . "to reput the chunk in place after gettextization. A "
-                  . "possible cause is that a text duplicated in the original "
-                  . "is not translated the same way each time. Remove one of "
-                  . "the translations, and you're fine."
-            ),
-            $poorig->count_entries_doc(),
-            $potrans->count_entries_doc()
-        );
-        $please_fail = 1;
-    }
-
-    if ( $poorig->get_charset =~ /^utf-8$/i ) {
-        $potrans->to_utf8;
-        $pores->set_charset("UTF-8");
-    } else {
-        my $charset = $potrans->get_charset();
-        $charset = "UTF-8" if $charset eq "CHARSET";
-        $pores->set_charset($charset);
-    }
-    print "Po character sets:\n"
-      . "  original="
-      . $poorig->get_charset . "\n"
-      . "  translated="
-      . $potrans->get_charset . "\n"
-      . "  result="
-      . $pores->get_charset . "\n"
-      if $debug{'encoding'};
-
-    for (
-        my ( $o, $t ) = ( 0, 0 ) ;
-        $o < $poorig->count_entries_doc() && $t < $potrans->count_entries_doc() ;
-        $o++, $t++
-      )
-    {
-        #
-        # Extract some informations
-
-        my ( $orig, $trans ) = ( $poorig->msgid_doc($o), $potrans->msgid_doc($t) );
-
-        #       print STDERR "Matches [[$orig]]<<$trans>>\n";
-
-        my ( $reforig,  $reftrans )  = ( $poorig->{po}{$orig}{'reference'}, $potrans->{po}{$trans}{'reference'} );
-        my ( $typeorig, $typetrans ) = ( $poorig->{po}{$orig}{'type'},      $potrans->{po}{$trans}{'type'} );
-
-        #
-        # Make sure the type of both string exist
-        #
-        die wrap_mod( "po4a gettextize", "Internal error: type of original string number %s " . "isn't provided", $o )
-          if ( $typeorig eq '' );
-
-        die wrap_mod( "po4a gettextize", "Internal error: type of translated string number %s " . "isn't provided", $o )
-          if ( $typetrans eq '' );
-
-        #
-        # Make sure both type are the same
-        #
-        if ( $typeorig ne $typetrans ) {
-            $pores->write("gettextization.failed.po");
-            eval {
-                # Recode $trans into current charset, if possible
-                require I18N::Langinfo;
-                I18N::Langinfo->import(qw(langinfo CODESET));
-                my $codeset = langinfo( CODESET() );
-                Encode::from_to( $trans, $potrans->get_charset, $codeset );
-            };
-            die wrap_msg(
-                dgettext( "po4a",
-                        "po4a gettextization: Structure disparity between "
-                      . "original and translated files:\n"
-                      . "msgid (at %s) is of type '%s' while\n"
-                      . "msgstr (at %s) is of type '%s'.\n"
-                      . "Original text: %s\n"
-                      . "Translated text: %s\n"
-                      . "(result so far dumped to gettextization.failed.po)" )
-                  . "%s",
-                $reforig,
-                $typeorig,
-                $reftrans,
-                $typetrans,
-                $orig, $trans, $toobad
-            );
-        }
-
-        #
-        # Push the entry
-        #
-        my $flags;
-        if ( defined $poorig->{po}{$orig}{'flags'} ) {
-            $flags = $poorig->{po}{$orig}{'flags'} . " fuzzy";
-        } else {
-            $flags = "fuzzy";
-        }
-        $pores->push_raw(
-            'msgid'     => $orig,
-            'msgstr'    => $trans,
-            'flags'     => $flags,
-            'type'      => $typeorig,
-            'reference' => $reforig,
-            'conflict'  => 1,
-            'transref'  => $potrans->{po}{$trans}{'reference'}
-          )
-          unless ( defined( $pores->{po}{$orig} )
-            and ( $pores->{po}{$orig}{'msgstr'} eq $trans ) )
-
-          # FIXME: maybe we should be smarter about what reference should be
-          #        sent to push_raw.
-    }
-
-    # make sure we return a useful error message when entry count differ
-    die "$toobad\n" if $please_fail;
-
-    return $pores;
-}
-
 =item filter($)
 
 This function extracts a catalog from an existing one. Only the entries having
@@ -1053,29 +906,6 @@ sub filter {
     return $res;
 }
 
-=item to_utf8()
-
-Recodes to UTF-8 the PO's msgstrs. Does nothing if the charset is not
-specified in the PO file ("CHARSET" value), or if it's already UTF-8 or
-ASCII.
-
-=cut
-
-sub to_utf8 {
-    my $this    = shift;
-    my $charset = $this->get_charset();
-
-    unless ( $charset eq "CHARSET"
-        or $charset =~ /^ascii$/i
-        or $charset =~ /^utf-8$/i )
-    {
-        foreach my $msgid ( keys %{ $this->{po} } ) {
-            Encode::from_to( $this->{po}{$msgid}{'msgstr'}, $charset, "utf-8" );
-        }
-        $this->set_charset("UTF-8");
-    }
-}
-
 =back
 
 =head1 Functions to use a message catalog for translations
@@ -1169,13 +999,7 @@ sub gettext {
     }
 
     if ( $opt{'wrap'} ) {
-        if ( $self->get_charset =~ /^utf-8$/i ) {
-            $res = Encode::decode_utf8($res);
-            $res = wrap( $res, $opt{'wrapcol'} || 76 );
-            $res = Encode::encode_utf8($res);
-        } else {
-            $res = wrap( $res, $opt{'wrapcol'} || 76 );
-        }
+        $res = wrap( $res, $opt{'wrapcol'} || 76 );
     }
 
     #    print STDERR "Gettext >>>$text<<<(escaped=$esc_text)=[[[$res]]]\n\n";
@@ -1356,12 +1180,6 @@ sub push_raw {
         #       } FIXME: do that iff the header isn't the default one.
         $self->{header}         = $msgstr;
         $self->{header_comment} = $comment;
-        my $charset = $self->get_charset;
-        if ( $charset ne "CHARSET" ) {
-            $self->{encoder} = find_encoding($charset);
-        } else {
-            $self->{encoder} = find_encoding("UTF-8");
-        }
         return;
     }
 
@@ -1401,14 +1219,17 @@ sub push_raw {
 
             if ($keep_conflict) {
                 if ( $self->{po}{$msgid}{'msgstr'} =~ m/^#-#-#-#-#  .*  #-#-#-#-#\\n/s ) {
-                    $msgstr = $self->{po}{$msgid}{'msgstr'} . "\\n#-#-#-#-#  $transref  #-#-#-#-#\\n" . $msgstr;
+                    $msgstr =
+                      $self->{po}{$msgid}{'msgstr'} . "\\n#-#-#-#-#  $transref (type: $type)  #-#-#-#-#\\n" . $msgstr;
                 } else {
                     $msgstr =
                         "#-#-#-#-#  "
                       . $self->{po}{$msgid}{'transref'}
-                      . "  #-#-#-#-#\\n"
+                      . " (type "
+                      . $self->{po}{$msgid}{'type'}
+                      . ")  #-#-#-#-#\\n"
                       . $self->{po}{$msgid}{'msgstr'} . "\\n"
-                      . "#-#-#-#-#  $transref  #-#-#-#-#\\n"
+                      . "#-#-#-#-#  $transref (type: $type)  #-#-#-#-#\\n"
                       . $msgstr;
                 }
 
@@ -1451,11 +1272,11 @@ sub push_raw {
     $self->{po}{$msgid}{'comment'}   = $comment;
     $self->{po}{$msgid}{'automatic'} = $automatic;
     $self->{po}{$msgid}{'previous'}  = $previous;
-    if ( defined( $self->{po}{$msgid}{'pos_doc'} ) ) {
-        $self->{po}{$msgid}{'pos_doc'} .= " " . $self->{count_doc}++;
-    } else {
-        $self->{po}{$msgid}{'pos_doc'} = $self->{count_doc}++;
-    }
+
+    $self->{po}{$msgid}{pos_doc} = () unless ( defined( $self->{po}{$msgid}{pos_doc} ) );
+    CORE::push( @{ $self->{po}{$msgid}{pos_doc} }, $self->{count_doc}++ );
+    CORE::push( @{ $self->{gettextize_types} },    $type );
+
     unless ( defined( $self->{po}{$msgid}{'pos'} ) ) {
         $self->{po}{$msgid}{'pos'} = $self->{count}++;
     }
@@ -1515,37 +1336,6 @@ sub count_entries_doc($) {
     return $self->{count_doc};
 }
 
-=item equals_msgid(po)
-
-Returns ($uptodate, $diagnostic) with $uptodate indicating whether all msgid of the current po file are 
-also present in the one passed as parameter (all other fields are ignored in the file comparison).
-Informally, if $uptodate returns false, then the po files would be changed when going through B<po4a-updatepo>.
-
-If $uptodate is false, then $diagnostic contains a diagnostic of why this is so.
-
-=cut
-
-sub equals_msgid($$) {
-    my ( $self, $other ) = ( shift, shift );
-
-    unless ( $self->count_entries() == $other->count_entries() ) {
-        return (
-            0,
-            wrap_msg(
-                dgettext( "po4a", "The amount of entries differ between files: %d is not %d" ),
-                $self->count_entries(),
-                $other->count_entries()
-            )
-        );
-    }
-    foreach my $msgid ( keys %{ $self->{po} } ) {
-        unless ( defined( $self->{po}{$msgid} ) && defined( $other->{po}{$msgid} ) ) {
-            return ( 0, wrap_msg( dgettext( "po4a", "msgid declared in one file only: %s\n" ), $msgid ) );
-        }
-    }
-    return ( 1, "" );
-}
-
 =item msgid($)
 
 Returns the msgid of the given number.
@@ -1573,13 +1363,29 @@ sub msgid_doc($$) {
     my $num  = shift;
 
     foreach my $msgid ( keys %{ $self->{po} } ) {
-        foreach my $pos ( split / /, $self->{po}{$msgid}{'pos_doc'} ) {
+        foreach my $pos ( @{ $self->{po}{$msgid}{'pos_doc'} } ) {
             return $msgid if ( $pos eq $num );
         }
     }
     return undef;
 }
 
+=item type_doc($)
+
+Returns the type of the msgid with the given position in the document. This is
+probably only useful to gettextization, and it's stored separately from
+{$msgid}{'type'} because the later location may be overwritten by another type
+when the $msgid is duplicated in the master document.
+
+=cut
+
+sub type_doc($$) {
+    my $self = shift;
+    my $num  = shift;
+
+    return ${ $self->{gettextize_types} }[$num];
+}
+
 =item get_charset()
 
 Returns the character set specified in the PO header. If it hasn't been
@@ -1599,29 +1405,6 @@ sub get_charset() {
     }
 }
 
-=item set_charset($)
-
-This sets the character set of the PO header to the value specified in its
-first argument. If you never call this function (and no file with a specified
-character set is read), the default value is left to "UTF-8". This value
-doesn't change the behavior of this module, it's just used to fill that field
-in the header, and to return it in get_charset().
-
-=cut
-
-sub set_charset() {
-    my $self = shift;
-
-    my ( $newchar, $oldchar );
-    $newchar = shift;
-    $oldchar = $self->get_charset();
-
-    if ( $newchar ne $oldchar ) {
-        $self->{header} =~ s/$oldchar/$newchar/;
-        $self->{encoder} = find_encoding($newchar);
-    }
-}
-
 #----[ helper functions ]---------------------------------------------------
 
 # transforme the string from its PO file representation to the form which
@@ -1689,7 +1472,7 @@ sub escape_text {
 #   on the 80th char, but without changing the meaning of the string)
 sub quote_text {
     my $string  = shift;
-    my $do_wrap = shift;    # either 'no' or 'newlines', or column at which we should wrap
+    my $do_wrap = shift // 'no';    # either 'no' or 'newlines', or column at which we should wrap
 
     return '""' unless length($string);
 
