@@ -199,7 +199,7 @@ and line number) afterward. I'm not sure what the best thing to do is.
 
 package Locale::Po4a::Sgml;
 
-use 5.006;
+use 5.16.0;
 use strict;
 use warnings;
 
@@ -271,10 +271,10 @@ sub initialize {
 }
 
 sub read {
-    my ( $self, $filename, $refname ) = @_;
+    my ( $self, $filename, $refname, $charset ) = @_;
 
     push @{ $self->{DOCPOD}{infile} }, $filename;
-    $self->Locale::Po4a::TransTractor::read( $filename, $refname );
+    $self->Locale::Po4a::TransTractor::read( $filename, $refname, $charset );
 }
 
 sub parse {
@@ -399,19 +399,18 @@ sub parse_file {
         my ( $tmpfh, $tmpfile ) = File::Temp::tempfile(
             "po4a-XXXX",
             SUFFIX => ".sgml",
-            DIR    => $ENV{TMPDIR} || "/tmp",
+            DIR    => File::Spec->tmpdir(),
             UNLINK => 0
         );
         print $tmpfh $origfile;
         close $tmpfh
-          or die wrap_mod( "po4a::sgml", dgettext( "po4a", "Can't close tempfile: %s" ), $! );
-        if ( system("onsgmls -p < $tmpfile") ) {
+          or die wrap_mod( "po4a::sgml", dgettext( "po4a", "Cannot close tempfile: %s" ), $! );
+        if ( system("onsgmls -p $tmpfile") ) {
             unlink($tmpfile);
             die wrap_mod(
                 "po4a::sgml",
                 dgettext(
-                    "po4a",
-                    "Error while running onsgmls -p.  " . "Please check if onsgmls and the " . "DTD are installed."
+                    "po4a", "Error while running onsgmls -p.  Please check if onsgmls and the DTD are installed."
                 )
             );
         }
@@ -435,7 +434,7 @@ sub parse_file {
     # Get the prolog
     {
         $prolog = $origfile;
-        my $lvl;    # number of '<' seen without matching '>'
+        my $lvl;        # number of '<' seen without matching '>'
         my $pos = 0;    # where in the document (in chars) while detecting prolog boundaries
 
         unless ( $prolog =~ s/^(.*<!DOCTYPE).*$/$1/is ) {
@@ -607,7 +606,7 @@ sub parse_file {
         my $pre     = scalar @begin;
         my $len     = ( scalar @ignored ) - 1;
         $pre++ if ( $begin =~ /\n$/s );
-        $len++ if ( $end =~ /^\n/s );
+        $len++ if ( $end   =~ /^\n/s );
 
         # remove the references of the ignored lines
         splice @refs, $pre + 1, $len - 1;
@@ -668,7 +667,7 @@ sub parse_file {
             } else {
                 $prolog = $begin . $end;
                 ( -e $filename && open IN, "<$filename" )
-                  || die wrap_mod( "po4a::sgml", dgettext( "po4a", "Can't open %s (content of entity %s%s;): %s" ),
+                  || die wrap_mod( "po4a::sgml", dgettext( "po4a", "Cannot open %s (content of entity %s%s;): %s" ),
                     $filename, '%', $key, $! );
                 local $/ = undef;
                 $prologentincl{$key} = <IN>;
@@ -752,7 +751,7 @@ sub parse_file {
 
             # Preload the content of the entity
             ( -e $filename && open IN, "<$filename" )
-              || die wrap_mod( "po4a::sgml", dgettext( "po4a", "Can't open %s (content of entity %s%s;): %s" ),
+              || die wrap_mod( "po4a::sgml", dgettext( "po4a", "Cannot open %s (content of entity %s%s;): %s" ),
                 $filename, '&', $key, $! );
             local $/ = undef;
             $entincl{$key}{'content'} = <IN>;
@@ -774,7 +773,7 @@ sub parse_file {
             while ( $origfile =~ /^(.*?)&$key(;.*$|[^-_:.A-Za-z0-9].*$|$)/s ) {
 
                 # Since we will include a new file, we
-                # must do a new round of substitutions.
+                # must do a new round of substitutions.
                 $dosubstitution = 1;
                 my ( $begin, $end ) = ( $1, $2 );
                 $end = "" unless ( defined $end );
@@ -790,7 +789,7 @@ sub parse_file {
                 # add the refs
                 my $len  = $entincl{$key}{'length'};    # number added by the inclusion
                 my $pre  = ( $begin =~ tr/\n/\n/ );     # number of \n
-                my $post = ( $end =~ tr/\n/\n/ );
+                my $post = ( $end   =~ tr/\n/\n/ );
                 print "XX Add a ref. pre=$pre; len=$len; post=$post\n"
                   if $debug{'refs'};
 
@@ -853,16 +852,16 @@ sub parse_file {
     my ( $tmpfh, $tmpfile ) = File::Temp::tempfile(
         "po4a-XXXX",
         SUFFIX => ".sgml",
-        DIR    => $ENV{TMPDIR} || "/tmp",
+        DIR    => File::Spec->tmpdir(),
         UNLINK => 0
     );
     print $tmpfh $origfile;
-    close $tmpfh or die wrap_mod( "po4a::sgml", dgettext( "po4a", "Can't close tempfile: %s" ), $! );
+    close $tmpfh or die wrap_mod( "po4a::sgml", dgettext( "po4a", "Cannot close tempfile: %s" ), $! );
 
-    my $cmd = "onsgmls -l -E 0 -wno-valid < $tmpfile" . ( $debug{'onsgmls'} ? "" : " 2>/dev/null" ) . " |";
+    my $cmd = "onsgmls -l -E 0 -wno-valid $tmpfile" . ( $debug{'onsgmls'} ? "" : " 2>/dev/null" ) . " |";
     print STDERR "CMD=$cmd\n" if ( $debug{'generic'} or $debug{'onsgmls'} );
 
-    open( IN, $cmd ) || die wrap_mod( "po4a::sgml", dgettext( "po4a", "Can't run onsgmls: %s" ), $! );
+    open( IN, $cmd ) || die wrap_mod( "po4a::sgml", dgettext( "po4a", "Cannot run onsgmls: %s" ), $! );
 
     # The kind of tags
     my ( %translate, %empty, %verbatim, %indent, %exist, %attribute, %qualify );
@@ -918,7 +917,7 @@ sub parse_file {
     # and push("<!ENTITY myentity \"".$self->translate("definition_of_my_entity")
     if ( $prolog =~ m/(.*?\[)(.*)(\]>)/s ) {
         warn "Pre=~~$1~~;Post=~~$3~~\n" if ( $debug{'entities'} );
-        $self->pushline( $1 . "\n" ) if ( length($1) );
+        $self->pushline( $1 . "\n" )    if ( length($1) );
         $prolog = $2;
         my ($post) = $3;
         while ( $prolog =~ m/^(.*?)<!ENTITY\s+(\S*)\s+"([^"]*)"\s*>(.*)$/is ) {    #" ){
@@ -931,7 +930,7 @@ sub parse_file {
         $self->pushline( $prolog . "\n" ) if ( length($prolog) );
     } else {
         warn "No entity declaration detected in ~~$prolog~~...\n" if ( $debug{'entities'} );
-        $self->pushline($prolog) if length($prolog);
+        $self->pushline($prolog)                                  if length($prolog);
     }
 
     # The parse object.
@@ -989,7 +988,9 @@ sub parse_file {
             # Which tag did we see?
             my $tag = '';
             $tag .= '<' . lc( $event->data->name() );
-            while ( my ( $attr, $val ) = each %{ $event->data->attributes() } ) {
+            foreach my $attr ( sort $event->data->attribute_names() ) {
+
+                my $val   = ${ $event->data->attributes() }{$attr};
                 my $value = $val->value();
 
                 #                if ($val->type() eq 'IMPLIED') {
@@ -1360,4 +1361,4 @@ The adaptation for po4a was done by:
  Copyright © 2002-2005 SPI, Inc.
 
 This program is free software; you may redistribute it and/or modify it
-under the terms of GPL (see the COPYING file).
+under the terms of GPL v2.0 or later (see the COPYING file).
