@@ -64,7 +64,7 @@ These are this module's particular options:
 
 package Locale::Po4a::TeX;
 
-use 5.006;
+use 5.16.0;
 use strict;
 use warnings;
 
@@ -86,7 +86,7 @@ use vars qw(@ISA @EXPORT);
 use Locale::Po4a::TransTractor;
 use Locale::Po4a::Common;
 use File::Basename qw(dirname);
-use Carp qw(croak);
+use Carp           qw(croak);
 
 use Encode;
 use Encode::Guess;
@@ -166,8 +166,7 @@ this environment does not take any parameters.
 
 =back
 
-Using these options permits to override the behaviour of the commands defined
-in the default lists.
+Use these options to override the default behavior of the defined commands.
 
 =head1 INLINE CUSTOMIZATION
 
@@ -184,7 +183,7 @@ treated as the arguments of the I<command2> command.
 
 =item B<% po4a: command> I<command1> I<parameters>
 
-This permit to describe in detail the parameters of the I<command1>
+This describes in detail the parameters of the I<command1>
 command.
 This information will be used to check the number of arguments and their
 types.
@@ -206,7 +205,7 @@ As for an asterisk, the command will be extracted if it appear at an
 extremity of a block, but the parameters won't be translated separately.
 The translator will have to translate the command concatenated to all its
 parameters.
-This permits to keep more context, and is useful for commands with small
+This keeps more context, and is useful for commands with small
 words in parameter, which can have multiple meanings (and translations).
 
 Note: In this case you don't have to specify which parameters are
@@ -244,9 +243,9 @@ command.
 
 =item B<% po4a: environment> I<env> I<parameters>
 
-This permits to define the parameters accepted by the I<env> environment.
+This defines the parameters accepted by the I<env> environment and specifies the ones to be translated.
 This information is later used to check the number of arguments of the
-\begin command, and permit to specify which one must be translated.
+\begin command.
 The syntax of the I<parameters> argument is the same as described for the
 others commands.
 The first parameter of the \begin command is the name of the environment.
@@ -264,7 +263,7 @@ Indicates that an environment should be split according to the given
 regular expression.
 
 The regular expression is delimited by quotes.
-It should not create any backreference.
+It should not create any back-reference.
 You should use (?:) if you need a group.
 It may also need some escapes.
 
@@ -927,15 +926,16 @@ Overloads Transtractor's read().
 
 =cut
 
-sub read {
+sub read ($$$$) {
     my $self     = shift;
     my $filename = shift;
     my $refname  = shift;
+    my $charset  = shift;
 
     # keep the directory name of the main file.
     $my_dirname = dirname($filename);
 
-    push @{ $self->{TT}{doc_in} }, read_file( $self, $filename, $refname );
+    push @{ $self->{TT}{doc_in} }, read_file( $self, $filename, $refname, $charset );
 }
 
 =item B<read_file>
@@ -955,10 +955,11 @@ sub read_file {
     my $filename = shift
       or croak wrap_mod( "po4a::tex", dgettext( "po4a", "Cannot read from file without having a filename" ) );
     my $refname = shift // $filename;
+    my $charset = shift || 'UTF-8';
     my $linenum = 0;
     my @entries = ();
 
-    open( my $in, $filename )
+    open( my $in, "<:encoding($charset)", $filename )
       or croak wrap_mod( "po4a::tex", dgettext( "po4a", "Cannot read from %s: %s" ), $filename, $! );
     while ( defined( my $textline = <$in> ) ) {
         $linenum++;
@@ -996,11 +997,19 @@ sub read_file {
                 open( KPSEA, "kpsewhich " . $newfilename . " |" );
                 my $newfilepath = <KPSEA>;
 
-                if ( $newfilename ne "" and $newfilepath eq "" ) {
-                    die wrap_mod( "po4a::tex", dgettext( "po4a", "Cannot find %s with kpsewhich" ), $filename );
+                if ( $newfilename ne "" and ( $newfilepath // '' ) eq '' ) {
+                    die wrap_mod(
+                        "po4a::tex",
+                        dgettext(
+                            "po4a",
+                            "Cannot find '%s' with kpsewhich. To prevent this file to be included, add '-o exclude_include=%s' to the options, either on the command line or in your po4a.conf file."
+                        ),
+                        $newfilename,
+                        $newfilename
+                    );
                 }
 
-                push @entries, read_file( $self, $newfilepath, $newfilename );
+                push @entries, read_file( $self, $newfilepath, $newfilename, $charset );
                 if ( $tag eq "include" ) {
                     $textline = "\\clearpage" . $end;
                 } else {
@@ -1011,20 +1020,6 @@ sub read_file {
         if ( length($textline) ) {
             my @entry = ( $textline, $ref );
             push @entries, @entry;
-
-            # Detect if this file has non-ascii characters
-            if ( $self->{TT}{ascii_input} ) {
-
-                my $decoder = guess_encoding($textline);
-                if ( !ref($decoder) or $decoder !~ /Encode::XS=/ ) {
-
-                    # We have detected a non-ascii line
-                    $self->{TT}{ascii_input} = 0;
-
-                    # Save the reference for future error message
-                    $self->{TT}{non_ascii_ref} ||= $ref;
-                }
-            }
         }
     }
     close $in
@@ -1121,9 +1116,9 @@ sub parse_definition_line {
         }
     } elsif ( $line =~ /^separator\s+(\w+(?:\[#[0-9]+\])?)\s+\"(.*)\"\s*$/ ) {
         my $env = $1;    # This is not necessarily an environment.
-                         # It can also be smth like 'title[#1]'.
+                         # It can also be something like 'title[#1]'.
         $env_separators{$env} = $2;
-    } elsif ( $line =~ /^verbatim\s+environment\s+(\w+)\s+$/ ) {
+    } elsif ( $line =~ /^verbatim\s+environment\s+(\w+)\s*$/ ) {
         register_verbatim_environment($1);
     }
 }
@@ -1721,7 +1716,7 @@ attached to the following string.
 =item Some commands should be added to the environment stack
 
 These commands should be specified by couples.
-This could allow to specify commands beginning or ending a verbatim
+This can be used to specify commands beginning or ending a verbatim
 environment.
 
 =item Others
@@ -1749,8 +1744,12 @@ L<po4a(7)|po4a.7>
 Copyright © 2004, 2005 Nicolas FRANÇOIS <nicolas.francois@centraliens.net>.
 
 This program is free software; you may redistribute it and/or modify it
-under the terms of GPL (see the COPYING file).
+under the terms of GPL v2.0 or later (see the COPYING file).
 
 =cut
 
 1;
+
+__END__
+
+#  LocalWords: Charset charset po UTF gettext msgid nostrip LaTeX
