@@ -60,13 +60,13 @@ L<po4a(7)|po4a.7>
 Copyright © 2004-2007 Nicolas FRANÇOIS <nicolas.francois@centraliens.net>.
 
 This program is free software; you may redistribute it and/or modify it
-under the terms of GPL (see COPYING file).
+under the terms of GPL v2.0 or later (see COPYING file).
 
 =cut
 
 package Locale::Po4a::Texinfo;
 
-use 5.006;
+use 5.16.0;
 use strict;
 use warnings;
 
@@ -336,7 +336,7 @@ sub translate_buffer_menuentry {
 
     my $translated_buffer = "";
 
-    if (   $buffer =~ m/^(.*?)(::)\s+(.*)$/s
+    if (   $buffer =~ m/^(.*?)(::)(?:\s+(.*))?$/s
         or $buffer =~ m/^(.*?: .*?)(\.)\s+(.*)$/s )
     {
         my ( $name, $sep, $description ) = ( $1, $2, $3 );
@@ -347,8 +347,9 @@ sub translate_buffer_menuentry {
             $translated_buffer .= ' ' x ( $menu_sep_width - 1 - $l );
             $l = $menu_sep_width - 1;
         }
-        ( $t, @e ) = $self->translate_buffer( $description, $no_wrap, @env );
-
+        if ($description) {
+            ( $t, @e ) = $self->translate_buffer( $description, $no_wrap, @env );
+        }
         # Replace newlines with space for proper wrapping
         # See https://github.com/mquinson/po4a/issues/122
         $t =~ s/\n/ /sg;
@@ -380,7 +381,7 @@ sub translate_buffer_ignore {
 $translate_buffer_env{"ignore"} = \&translate_buffer_ignore;
 
 foreach (
-    qw(appendix section cindex findex kindex opindex pindex vindex subsection
+    qw(appendix section cindex findex kindex opindex pindex tindex vindex subsection
     dircategory subtitle include
     exdent center unnumberedsec
     heading unnumbered unnumberedsubsec
